@@ -4,7 +4,7 @@ use warnings; use strict; use utf8;
 #use 5.010; # Needed for smart-match operator
 #v5.10 feature use removed until 5.10 is standard on MacOSX and Debian
 use English qw( -no_match_vars );
-use version 0.74; our $VERSION = qv("0.4.6");
+use version 0.74; our $VERSION = qv("0.4.8");
 
 # Perl Critic overrides:
 ## no critic (Package variable)
@@ -131,6 +131,9 @@ our %exthtypes = (
     205 => '205',
     206 => '206',
     207 => '207',
+    208 => '208',
+    209 => '209',
+    210 => '210',
     300 => '300',
     401 => 'clippinglimit',
     402 => 'publisherlimit',
@@ -658,7 +661,7 @@ sub ParseRecord :method   ## no critic (Always unpack @_ first)
         else
         {
             debug(3,"DEBUG: record ",$currentrecord,
-                  " is PalmDoc-compressed text");
+                  " is uncompressed or PalmDoc-compressed text");
             $self->ParseRecordText(\$data);
         }
     }
@@ -1217,14 +1220,41 @@ sub ParseRecordText :method
     my $currentrecord = scalar @{$$self{records}};
     my $compression = $$self{header}{palm}{compression};
     my $recordtext;
+    my $extradatasize;
+
+    debug(1,"DEBUG: extradataflags = ",$$self{header}{mobi}{extradataflags});
+    if( $$self{header}{mobi}{extradataflags} )
+    {
+        $extradatasize = record_extradata_size(
+            dataref => $dataref,
+            extradataflags => $$self{header}{mobi}{extradataflags}
+           );
+    }
 
     if($compression == 1)        # No compression
     {
-        $recordtext = $$dataref;
+        debug(3,"DEBUG: No compression on record ",$currentrecord);
+        if($extradatasize)
+        {
+            $recordtext = substr($$dataref,0,length($$dataref)-$extradatasize);
+            debug(3,"DEBUG: skipping ",$extradatasize," bytes at end of record ",
+                  $currentrecord);
+        }
+        else
+        {
+            $recordtext = $$dataref;
+        }
     }
     elsif($compression == 2)     # PalmDoc compression
     {
-        $recordtext = uncompress_palmdoc($$dataref);
+        my %args;
+        if($extradatasize)
+        {
+            $args{trailing} = $extradatasize;
+            debug(3,"DEBUG: skipping ",$extradatasize," bytes at end of record ",
+                  $currentrecord);
+        }
+        $recordtext = uncompress_palmdoc($$dataref,%args);
     }
     elsif($compression == 17480) # 'Dictionary Huffman' compression
     {
@@ -1243,12 +1273,12 @@ sub ParseRecordText :method
     if($recordtext)
     {
         $self->{text} .= $recordtext;
-        debug(3,"DEBUG: uncompressed text record ",$currentrecord);
+        debug(3,"DEBUG: parsed text record ",$currentrecord);
     }
     else
     {
         debug(1,"DEBUG: record ",$currentrecord,
-              " could not be decompressed (",
+              " could not be parsed (",
               length($$dataref)," bytes starting with '",
               substr($$dataref,0,4),"')");
         $$self{unknowndata}{$currentrecord} = $$dataref;
@@ -2096,6 +2126,12 @@ contain it.
 Use with caution.  This key may be renamed in the future if more
 information is found.
 
+=item C<extradataflags>
+
+Two bytes sometimes found inside of C<unknown200>, used to determine
+if extra data has been appended to each text record that should not be
+used in decompression.
+
 =back
 
 =cut
@@ -2263,6 +2299,19 @@ sub parse_mobi_header   ## no critic (ProhibitExcessComplexity)
         debug(2,"       0x",hexstring($header{unknown200}));
     }
 
+    # Part of that unknown data is the Extra Data Flags
+    #
+    # This is 16 bits used to determine if extra data has been stuffed
+    # at the end of each record that should not be used in
+    # decompression.
+    if($length >= 228)
+    {
+        $chunk = substr($headerdata,226,2);
+        $header{extradataflags} = unpack("n",$chunk);
+        debug(2,"DEBUG: Found Extra Data Flags in Mobipocket header");
+        debug(2,"       0x",hexstring($chunk));
+    }
+
     foreach my $key (sort keys %header)
     {
         no warnings;
@@ -2517,6 +2566,103 @@ sub pukall_cipher_1
 }
 
 
+=head2 C<record_extradata_size(%args)>
+
+This checks the end of a text record for extra data that should not be
+made part of decompression and returns the total size of all data
+fields.
+
+=head3 Arguments
+
+=over
+
+=item * C<dataref>
+
+A reference to the record data
+
+=item * C<extradataflags>
+
+16 bits worth of flags indicating which extra data fields are present.
+
+=back
+
+=cut
+
+sub record_extradata_size
+{
+    my %args = @_;
+    my $subname = ( caller(0) )[3];
+    debug(3,"DEBUG[",$subname,"]");
+
+    my %valid_args = (
+        'dataref' => 1,
+        'extradataflags' => 1,
+        );
+    foreach my $arg (keys %args)
+    {
+        croak($subname,"(): invalid argument '",$arg,"'")
+            if(!$valid_args{$arg});
+    }
+
+    my $dataref = $args{dataref};
+    croak($subname,"(): no record data provided!\n")
+        unless(defined $dataref);
+    croak($subname,"(): record data is not a reference\n") unless(ref $dataref);
+    croak($subname,"(): no extra data flags provided!\n")
+        unless(defined $args{extradataflags});
+
+    my $datalength = length($$dataref);
+
+    my $totalsize = 0;
+    my $trailpos = 0;
+
+    foreach my $flagbit (reverse 0..15)
+    {
+        if($args{extradataflags} & (1 << $flagbit))
+        {
+            my $bitpos = 0;
+            my $startpos = $trailpos;
+            my $traildata;
+            my $trailsize = 0;
+            my $byte;
+
+            # Bit 0 is the multi-byte character overlap flag, and has
+            # a different format from all other flags, where the size
+            # is the first two bits of the last unparsed byte of
+            # record data (i.e. the extra data bytes closest to the
+            # actual record text), plus one for the byte containing
+            # the size itself.
+            if($flagbit == 0)
+            {
+                $trailpos += 1;
+                $trailsize = ord(substr($$dataref,$datalength-$trailpos,1)) & 0x03;
+                $trailsize += 1; # The above line doesn't include the size byte itself
+                debug(3,"DEBUG: ",$trailsize," bytes of trailing data at flagbit ",
+                      $flagbit,", startpos ",$startpos);
+                $totalsize += $trailsize;
+                last;
+            }
+
+            # For all other bits, the size is a backward-encoded
+            # variable-width integer at the end of the record data.
+            do
+            {
+                $trailpos += 1;
+                $byte = ord(substr($$dataref,$datalength-$trailpos,1));
+                $trailsize |= (($byte & 0x7f) << $bitpos);
+                $bitpos += 7;
+            }
+            while( !($byte & 0x80) && ($bitpos < 28) && ($trailpos < $datalength) );
+            $traildata = substr($$dataref,$datalength-$trailsize,$trailsize);
+            debug(3,"DEBUG: ",$trailsize," bytes of trailing data at flagbit ",
+                  $flagbit,", startpos ",$startpos);
+            $totalsize += $trailsize;
+        }
+    }
+    return $totalsize;
+}
+
+
 =head2 C<system_mobidedrm(%args)>
 
 Runs python on a copy of C<MobiDeDrm.py> if it is available (not
